@@ -7,7 +7,7 @@
 %%   file, You can obtain one at http://mozilla.org/MPL/2.0/.
 %%
 %% -------------------------------------------------------------------
-%% @doc Internal reporter exposing metrics over SNMP.
+%% @doc Internal reporter exposing cloudi_x_metrics over SNMP.
 %%
 %% @end
 -module(cloudi_x_exometer_report_snmp).
@@ -39,7 +39,7 @@
 -export_type([snmp/0, snmp_option/0]).
 
 -include_lib("cloudi_x_exometer_core/include/cloudi_x_exometer.hrl").
--include("cloudi_x_log.hrl").
+-include("cloudi_x_exometer_log.hrl").
 
 -define(MIB_TEMPLATE, "mibs/EXOMETER-METRICS-MIB.mib").
 -define(MIB_DIR, "tmp/" ++ erlang:atom_to_list(?MODULE)).
@@ -77,13 +77,13 @@ exometer_init(Opts) ->
             ?warning("Application SNMP not started. Ensure that a usable SNMP agent is configured.")
     end,
 
-    % prepare nr mapping used to track enabled metrics
+    % prepare nr mapping used to track enabled cloudi_x_metrics
     ?MIB_NR_MAP = ets:new(?MIB_NR_MAP, [named_table]),
     ets:insert(?MIB_NR_MAP, {?MIB_NR_NEXT, 0}),
     ets:insert(?MIB_NR_MAP, {?MIB_NR_FREE, []}),
 
     % load MIB template which is used through the operation of
-    % the process to dynamically export metrics
+    % the process to dynamically export cloudi_x_metrics
     MibPath0 = proplists:get_value(mib_template, Opts, ?MIB_TEMPLATE),
     MibWorkPath = proplists:get_value(mib_dir, Opts, ?MIB_DIR),
     MibPath1 = filename:join([MibWorkPath, filename:basename(MibPath0)]),
