@@ -29,9 +29,9 @@ stop(_) ->
 -spec profile_output() -> ok.
 profile_output() ->
 	eprof:stop_profiling(),
-	eprof:cloudi_x_log("procs.profile"),
+	eprof:log("procs.profile"),
 	eprof:analyze(procs),
-	eprof:cloudi_x_log("total.profile"),
+	eprof:log("total.profile"),
 	eprof:analyze(total).
 
 consider_profiling() ->
