@@ -24,45 +24,31 @@
 %%% due to the transaction id being globally unique.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2015-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2015-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_queue).
@@ -103,8 +89,7 @@
         failures_source_max_count :: pos_integer(),
         failures_source_max_period :: infinity | pos_integer(),
         failures_source = [] :: list(erlang:timestamp()),
-        requests = dict:new() :: dict_proxy(cloudi_service:trans_id(),
-                                            #request{})
+        requests = #{} :: #{cloudi_service:trans_id() := #request{}}
     }).
 
 -include("cloudi_service.hrl").
@@ -246,8 +231,7 @@ recv(Dispatcher,
                    failures_source = FailuresSrc,
                    requests = Requests} = State)
     when is_pid(Dispatcher) ->
-    #request{} = dict:fetch(TransId, Requests),
-    NewRequests = dict:erase(TransId, Requests),
+    {#request{}, NewRequests} = maps:take(TransId, Requests),
     case validate(ResponseInfoF, ResponseF,
                   ResponseInfo, Response) of
         true ->
@@ -376,9 +360,9 @@ send(Dispatcher, Name, RequestInfo, Request, Timeout, Priority, PatternPid,
                                             timeout = Timeout,
                                             priority = Priority,
                                             pattern_pid = PatternPid},
-                    {ok, State#cloudi_queue{requests = dict:store(TransId,
-                                                                  RequestState,
-                                                                  Requests)}};
+                    {ok, State#cloudi_queue{requests = maps:put(TransId,
+                                                                RequestState,
+                                                                Requests)}};
                 {error, _} = Error ->
                     NewFailuresSrc = failure(FailuresSrcDie,
                                              FailuresSrcMaxCount,
@@ -419,7 +403,7 @@ timeout(Dispatcher,
                       failures_source = FailuresSrc,
                       requests = Requests} = State)
     when is_pid(Dispatcher) ->
-    case dict:fetch(TransId, Requests) of
+    case maps:get(TransId, Requests) of
         #request{retry_count = RetryMax} ->
             NewFailuresSrc = failure(FailuresSrcDie,
                                      FailuresSrcMaxCount,
@@ -427,7 +411,7 @@ timeout(Dispatcher,
                                      FailuresSrc),
             {{error, timeout},
              State#cloudi_queue{failures_source = NewFailuresSrc,
-                                requests = dict:erase(TransId, Requests)}};
+                                requests = maps:remove(TransId, Requests)}};
         #request{name = Name,
                  request_info = RequestInfo,
                  request = Request,
@@ -441,10 +425,10 @@ timeout(Dispatcher,
                                                   Timeout, Priority,
                                                   TransId, PatternPid) of
                 {ok, TransId} ->
-                    NewRequests = dict:store(TransId,
-                                             RequestState#request{
-                                                 retry_count = I + 1},
-                                             Requests),
+                    NewRequests = maps:put(TransId,
+                                           RequestState#request{
+                                               retry_count = I + 1},
+                                           Requests),
                     {ok, State#cloudi_queue{requests = NewRequests}};
                 {error, _} = Error ->
                     NewFailuresSrc = failure(FailuresSrcDie,
@@ -453,8 +437,8 @@ timeout(Dispatcher,
                                              FailuresSrc),
                     {Error,
                      State#cloudi_queue{failures_source = NewFailuresSrc,
-                                        requests = dict:erase(TransId,
-                                                              Requests)}}
+                                        requests = maps:remove(TransId,
+                                                               Requests)}}
             end
     end.
 
@@ -479,9 +463,9 @@ failure(false, _, _, FailureList) ->
 failure(true, MaxCount, MaxPeriod, FailureList) ->
     failure_check(cloudi_timestamp:seconds(), FailureList, MaxCount, MaxPeriod).
 
-failure_store(FailureList, MaxCount) ->
+failure_store(FailureList, FailureCount, MaxCount) ->
     if
-        erlang:length(FailureList) == MaxCount ->
+        FailureCount == MaxCount ->
             failure_kill();
         true ->
             ok
@@ -489,11 +473,15 @@ failure_store(FailureList, MaxCount) ->
     FailureList.
 
 failure_check(SecondsNow, FailureList, MaxCount, infinity) ->
-    failure_store([SecondsNow | FailureList], MaxCount);
+    NewFailureCount = erlang:length(FailureList),
+    failure_store([SecondsNow | FailureList], NewFailureCount + 1,
+                  MaxCount);
 failure_check(SecondsNow, FailureList, MaxCount, MaxPeriod) ->
-    NewFailureList = cloudi_timestamp:seconds_filter(FailureList,
-                                                     SecondsNow, MaxPeriod),
-    failure_store([SecondsNow | NewFailureList], MaxCount).
+    {NewFailureCount,
+     NewFailureList} = cloudi_timestamp:seconds_filter(FailureList,
+                                                       SecondsNow, MaxPeriod),
+    failure_store([SecondsNow | NewFailureList], NewFailureCount + 1,
+                  MaxCount).
 
 failure_kill() ->
     erlang:exit(cloudi_queue).
