@@ -7,45 +7,31 @@
 %%% A module that exposes dynamic configuration of CloudI.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2011-2017, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
 %%% @copyright 2011-2017 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_api).
@@ -71,6 +57,7 @@
          nodes/1,
          logging_set/2,
          logging_file_set/2,
+         logging_stdout_set/2,
          logging_level_set/2,
          logging_syslog_set/2,
          logging_formatters_set/2,
@@ -371,6 +358,8 @@
     list({name, nonempty_string()} |
          {parameters, list({nonempty_string(), string()})} |
          {update_or_create, boolean()}).
+-type chroot_external() ::
+    file:filename() | undefined.
 -type directory_external() ::
     file:filename() | undefined.
 -export_type([limit_external_key/0,
@@ -379,6 +368,7 @@
               owner_external/0,
               nice_external/0,
               cgroup_external/0,
+              chroot_external/0,
               directory_external/0]).
 
 -type service_options_internal() ::
@@ -525,6 +515,9 @@
          {aspects_terminate_before, list(aspect_terminate_before_external())} |
          {limit, limit_external()} |
          {owner, owner_external()} |
+         {nice, nice_external()} |
+         {cgroup, cgroup_external()} |
+         {chroot, chroot_external()} |
          {directory, directory_external()}).
 -export_type([service_options_internal/0,
               service_options_external/0]).
@@ -752,7 +745,7 @@
          Function :: atom() | undefined,
          Arity :: arity() | undefined,
          MetaData :: list({atom(), any()}),
-         LogMessage :: string()) ->
+         LogMessage :: iodata()) ->
         ok).
 -type aspect_log_before() ::
     aspect_log_f() |
@@ -805,6 +798,7 @@
                {formatter_config, list()})}).
 -type logging_proplist() ::
     nonempty_list({file, string() | undefined} |
+                  {stdout, boolean()} |
                   {level, loglevel()} |
                   {redirect, node() | undefined} |
                   {syslog, logging_syslog_set_proplist() | undefined} |
@@ -1286,6 +1280,24 @@ logging_file_set(FilePath, Timeout)
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging_file_set(FilePath, Timeout).
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Modify the current log stdout usage.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec logging_stdout_set(Stdout :: boolean(),
+                         Timeout :: api_timeout_milliseconds()) ->
+    ok.
+
+logging_stdout_set(Stdout, Timeout)
+    when is_boolean(Stdout),
+         ((is_integer(Timeout) andalso
+           (Timeout > ?TIMEOUT_DELTA) andalso
+           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+          (Timeout =:= infinity)) ->
+    cloudi_core_i_configurator:logging_stdout_set(Stdout, Timeout).
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Modify the current loglevel.===
