@@ -18,45 +18,31 @@
 %%% the traversals preserve alphabetical ordering.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2010-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
+%%% MIT License
 %%%
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
+%%% Copyright (c) 2010-2017 Michael Truog <mjtruog at gmail dot com>
 %%%
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
 %%%
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2010-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2010-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_trie).
@@ -104,6 +90,7 @@
          size/1,
          store/2,
          store/3,
+         take/2,
          to_list/1,
          to_list_similar/2,
          update/3,
@@ -839,19 +826,32 @@ pattern_parse(Pattern, L) ->
 
 -spec pattern_parse(Pattern :: string(),
                     L :: string(),
-                    Option :: default | with_suffix) ->
-    list(string()) | {list(string()), string()} | 'error'.
+                    Option :: default | with_suffix | expanded) ->
+    list(string()) |                       % default
+    {list(string()), string()} |           % with_suffix
+    list(string() | {exact, string()}) |   % expanded
+    'error'.
 
 pattern_parse(Pattern, L, Option)
     when (Option =:= default) orelse
-         (Option =:= with_suffix) ->
+         (Option =:= with_suffix) orelse
+         (Option =:= expanded) ->
     pattern_parse(Pattern, L, [], [], Option).
 
 pattern_parse_result(default, Parameters, _) ->
-    Parameters;
+    lists:reverse(Parameters);
 
 pattern_parse_result(with_suffix, Parameters, Suffix) ->
-    {Parameters, lists:reverse(Suffix)}.
+    {lists:reverse(Parameters), lists:reverse(Suffix)};
+
+pattern_parse_result(expanded, Parameters, Suffix) ->
+    NewParameters = if
+        Suffix /= [] ->
+            [{exact, lists:reverse(Suffix)} | Parameters];
+        true ->
+            Parameters
+    end,
+    lists:reverse(NewParameters).
 
 pattern_parse_element(_, [], _) ->
     error;
@@ -882,7 +882,7 @@ pattern_parse_pattern(Pattern, C, L, Segment, Parsed, Option) ->
     end.
 
 pattern_parse([], [], Parsed, Suffix, Option) ->
-    pattern_parse_result(Option, lists:reverse(Parsed), Suffix);
+    pattern_parse_result(Option, Parsed, Suffix);
 
 pattern_parse([], [_ | _], _, _, _) ->
     error;
@@ -890,14 +890,26 @@ pattern_parse([], [_ | _], _, _, _) ->
 pattern_parse([_ | _], [$* | _], _, _, _) ->
     erlang:exit(badarg);
 
-pattern_parse([$*], [_ | _] = L, Parsed, _, Option) ->
-    pattern_parse_result(Option, lists:reverse([L | Parsed]), []);
+pattern_parse([$*], [_ | _] = L, Parsed, Suffix, Option) ->
+    NewParsed = if
+        Option =:= expanded, Suffix /= [] ->
+            [{exact, lists:reverse(Suffix)} | Parsed];
+        true ->
+            Parsed
+    end,
+    pattern_parse_result(Option, [L | NewParsed], []);
 
 pattern_parse([$*, $* | _], [_ | _], _, _, _) ->
     erlang:exit(badarg);
 
-pattern_parse([$*, C | Pattern], [H | T], Parsed, _, Option) ->
-    pattern_parse_pattern(Pattern, C, T, [H], Parsed, Option);
+pattern_parse([$*, C | Pattern], [H | T], Parsed, Suffix, Option) ->
+    NewParsed = if
+        Option =:= expanded, Suffix /= [] ->
+            [{exact, lists:reverse(Suffix)} | Parsed];
+        true ->
+            Parsed
+    end,
+    pattern_parse_pattern(Pattern, C, T, [H], NewParsed, Option);
 
 pattern_parse([C | Pattern], [C | L], Parsed, Suffix, Option) ->
     pattern_parse(Pattern, L, Parsed, [C | Suffix], Option);
@@ -1145,6 +1157,9 @@ test() ->
     2.5 = cloudi_x_trie:fetch("aaaa", RootNode5),
     {'EXIT', {if_clause, _}} = (catch cloudi_x_trie:fetch("aaaa", RootNode4)),
     RootNode4 = cloudi_x_trie:erase("a", cloudi_x_trie:erase("aaaa", RootNode5)),
+    {2.5, RootNodePre4} = cloudi_x_trie:take("aaaa", RootNode5),
+    {0, RootNode4} = cloudi_x_trie:take("a", RootNodePre4),
+    error = cloudi_x_trie:take("a", RootNode4),
     true = cloudi_x_trie:is_key("aaaa", RootNode5),
     false = cloudi_x_trie:is_key("aaaa", RootNode4),
     ["aa",
@@ -1236,6 +1251,13 @@ test() ->
     {ok,"*/",empty} = cloudi_x_trie:find_match("///", cloudi_x_trie:new(["*/"])),
     [] = cloudi_x_trie:pattern_parse("aaabb", "aaabb"),
     {[], "aaabb"} = cloudi_x_trie:pattern_parse("aaabb", "aaabb", with_suffix),
+    [{exact, "a"},
+     "ddi",
+     {exact, "t"},
+     "io",
+     {exact, "n"}] = cloudi_x_trie:pattern_parse("a*t*n", "addition", expanded),
+    ["w",{exact,"atch"}] = cloudi_x_trie:pattern_parse("*atch", "watch", expanded),
+    [{exact,"is"},"t"] = cloudi_x_trie:pattern_parse("is*", "ist", expanded),
     false = cloudi_x_trie:is_pattern("abcdef"),
     true = cloudi_x_trie:is_pattern("abc*d*ef"),
     {'EXIT',badarg} = (catch cloudi_x_trie:is_pattern("abc**ef")),
