@@ -1,41 +1,27 @@
 /* -*-Mode:C;coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
  * ex: set ft=c fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
  *
- * BSD LICENSE
- * 
- * Copyright (c) 2011-2015, Michael Truog <mjtruog at gmail dot com>
- * All rights reserved.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *     * Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in
- *       the documentation and/or other materials provided with the
- *       distribution.
- *     * All advertising materials mentioning features or use of this
- *       software must display the following acknowledgment:
- *         This product includes software developed by Michael Truog
- *     * The name of the author may not be used to endorse or promote
- *       products derived from this software without specific prior
- *       written permission
- * 
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
- * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
- * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
- * DAMAGE.
+ * MIT License
+ *
+ * Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 #include "cloudi.h"
 #include <string.h>
@@ -49,8 +35,7 @@ typedef struct
 
 } process_requests_t;
 
-static void request(cloudi_instance_t * api,
-                    int const command,
+static void request(int const request_type,
                     char const * const name,
                     char const * const pattern,
                     void const * const request_info,
@@ -61,10 +46,12 @@ static void request(cloudi_instance_t * api,
                     int8_t priority,
                     char const * const trans_id,
                     char const * const pid,
-                    uint32_t const pid_size)
+                    uint32_t const pid_size,
+                    void * state,
+                    cloudi_instance_t * api)
 {
     char response[128];
-    char const ** http_qs = cloudi_request_http_qs_parse(request, request_size);
+    char const ** http_qs = cloudi_info_key_value_parse(request, request_size);
     char const * value = 0;
     size_t i;
     for (i = 0; http_qs[i]; i += 2)
@@ -75,7 +62,7 @@ static void request(cloudi_instance_t * api,
             break;
         }
     }
-    cloudi_request_http_qs_destroy(http_qs);
+    cloudi_info_key_value_destroy(http_qs);
     if (value)
     {
         snprintf(response, sizeof(response),
@@ -86,7 +73,7 @@ static void request(cloudi_instance_t * api,
         memcpy(response,
                "<http_test><error>no value specified</error></http_test>", 57);
     }
-    cloudi_return(api, command, name, pattern, "", 0,
+    cloudi_return(api, request_type, name, pattern, "", 0,
                   response, strlen(response),
                   timeout, trans_id, pid, pid_size);
 }
@@ -95,7 +82,7 @@ void process_requests(void * p)
 {
     cloudi_instance_t api;
     process_requests_t * data = (process_requests_t *) p;
-    int result = cloudi_initialize(&api, data->thread_index);
+    int result = cloudi_initialize(&api, data->thread_index, 0);
     assert(result == cloudi_success);
 
     result = cloudi_subscribe_count(&api, "c.xml/get");
