@@ -6,45 +6,31 @@
 %%% ==CloudI Service for the request_rate Test==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2014-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2014-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_request_rate).
@@ -60,47 +46,6 @@
 -include_lib("cloudi_core/include/cloudi_logger.hrl").
 -include_lib("cloudi_core/include/cloudi_service.hrl").
 
-%% Example Usage:
-
-% Hardware:
-%  Core i7 2670QM 2.2GHz 4 cores, 8 hyper-threads
-%  L2:4×256KB L3:6MB RAM:8GB:DDR3-1333MHz
-%  Sandy Bridge-HE-4 (Socket G2)
-%
-% Software:
-%  Ubuntu 12.04.2 LTS (GNU/Linux 3.2.0-29-generic x86_64)
-%  Erlang R16B03-1 (with lock counting, so probably slower than possible)
-%
-%
-% Basic Speed Test:
-% Max Stable (Total) Request Rate (during informal testing):
-%  requests/second dest_refresh_method duo_mode sender receiver
-%             1000           immediate    false      1        1
-%            12000           immediate     true      1        1
-%            13000                lazy     true      1        1
-%            20000                lazy     true      2        4
-%            27000                lazy     true      3        6 (overheating)
-%
-% (dest_refresh_method is set on cloudi_service_request_rate
-%  duo_mode is set on cloudi_service_request_rate when true
-%  sender is the number of cloudi_service_request_rate processes while
-%  receiver is the number of cloudi_service_http_req processes)
-%
-% Basic Memory Test:
-% Max Stable (Total) Request Rate (during informal testing):
-%  requests/second dest_refresh_method duo_mode sender receiver queue_size(kB)
-%              149           immediate    false      1        1            512 
-%             1049           immediate    false      1        1           4096
-%             1125           immediate    false      1        1         131072
-%               97                lazy     true      1        1            512 
-%              772                lazy     true      1        1           4096
-%             8128                lazy     true      1        1         131072
-%
-%  requests/second dest_refresh_method duo_mode sender receiver    queue_limit
-%                1           immediate    false      1        1              0
-%                1                lazy     true      1        1              0
-% 
-
 -define(DEFAULT_SERVICE_NAME,   "/tests/http_req/erlang.xml/get").
 -define(DEFAULT_REQUEST_INFO,             <<>>).
 -define(DEFAULT_REQUEST,        <<(<<"value">>)/binary, 0:8,
@@ -125,8 +70,6 @@
 -define(DEFAULT_TICK_LENGTH,              5000). % ms (set as async timeout)
 -define(DEFAULT_TICK_STABLE_COUNT,          24). % dynamic attempts for stable
 
--type dict_proxy(Key, Value) :: dict:dict(Key, Value).
-
 -record(dynamic,
     {
         count_stable_max :: pos_integer(),
@@ -147,7 +90,7 @@
         request_rate :: pos_integer() | #dynamic{},
         request_success :: non_neg_integer(),
         request_fail :: non_neg_integer(),
-        request_ids :: dict_proxy(cloudi_service:trans_id(), undefined),
+        request_ids :: #{cloudi_service:trans_id() := undefined},
         tick_length :: pos_integer()
     }).
 
@@ -220,7 +163,7 @@ cloudi_service_init(Args, _Prefix, _Timeout, Dispatcher) ->
                 request_rate = RequestRateN,
                 request_success = 0,
                 request_fail = 0,
-                request_ids = dict:new(),
+                request_ids = #{},
                 tick_length = TickLength}}.
 
 cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
@@ -232,7 +175,7 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                                   request_fail = RequestFail,
                                   request_ids = RequestIds} = State,
                            _Dispatcher) ->
-    case dict:find(TransId, RequestIds) of
+    case maps:find(TransId, RequestIds) of
         {ok, _} ->
             case validate(ValidateResponseInfo, ValidateResponse,
                           ResponseInfo, Response) of
@@ -454,7 +397,7 @@ tick_request_send(I, Name, RequestInfo, Request, Dispatcher) ->
         true ->
             Request
     end,
-    tick_request_send(I, dict:new(), Name,
+    tick_request_send(I, #{}, Name,
                       RequestInfoData, RequestData, Dispatcher).
 
 tick_request_send(0, RequestIds, _, _, _, _) ->
@@ -466,7 +409,7 @@ tick_request_send(I, RequestIds, Name,
                                                           undefined,
                                                           undefined) of
         {ok, TransId} ->
-            dict:store(TransId, undefined, RequestIds);
+            maps:put(TransId, undefined, RequestIds);
         {error, _} ->
             RequestIds
     end,
